# Duo, A crazy eights game.
# Copyright (C) 2003-2008 The Duo developers.
#
# This file is part of Duo.
#
# Duo is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""General game information and types.
"""

import random


class Card(object):
    """The Duo card representation."""

    COLORS = {'Y': 'Yellow', 'G': 'Green', 'R': 'Red', 'B': 'Blue'}

    TYPES = [
        '0',
        '1',
        '2',
        '3',
        '4',
        '5',
        '6',
        '7',
        '8',
        '9',
        'R',
        'P',
        'D',
        'W',
        'X']

    POINTS = {
        '0': 0,
        '1': 1,
        '2': 2,
        '3': 3,
        '4': 4,
        '5': 5,
        '6': 6,
        '7': 7,
        '8': 8,
        '9': 9,
        'R': 20,
        'P': 20,
        'D': 20,
        'W': 50,
        'X': 50
        }

    def __init__(self, type_, color='N'):
        self._color = color
        self._type = type_

    def __eq__(self, card):
        if card is None:
            return False
        else:
            return self._type == card.type and self._color == card.color

    def __ne__(self, card):
        return not self.__eq__(card)

    def __repr__(self):
        return self._type + self._color

    def __str__(self):
        return self._type + self._color

    def _get_color(self):
        """Retrieve the color of this card."""
        return self._color

    def _get_type(self):
        """Retrieve the type of this card."""
        return self._type

    color = property(_get_color, None, None, 'The card color')
    type = property(_get_type, None, None, 'The card type')


class Deck(list):
    """The Duo deck representation."""

    def __init__(self):
        list.__init__(self)

    def generate(self):
        """Fill this deck with a full non sorted cardset."""
        # Generate cards without color
        for type_ in Card.TYPES[13:]:
            self.append(Card(type_))
            self.append(Card(type_))
            self.append(Card(type_))
            self.append(Card(type_))

        # Generate colored numbered cards
        for color in Card.COLORS.keys():
            self.append(Card(Card.TYPES[0], color))
            for type_ in Card.TYPES[1:13]:
                self.append(Card(type_, color))
                self.append(Card(type_, color))

    def peek(self):
        """Retrieve the card on top of this deck, or None."""
        if len(self):
            return self[len(self) -1]
        else:
            return None

    def shuffle(self):
        """Shuffle the content of this deck."""
        random.shuffle(self)


class Kitty(list):
    """The Duo penalty stock representation."""

    def __init__(self):
        list.__init__(self)
        self._taken = False
        self._type = 'N'

    def pick_up(self):
        """Return the kitty content and mark it as taken."""
        cards = list(self[:])
        self[:] = []
        self._taken = True
        return cards
        
    def _get_type(self):
        """Return the type of the card which started the penalty."""
        return self._type

    def _set_type(self, type_):
        """Change the type of the card which started the penalty."""
        self._type = type_

    taken = property(lambda self: self._taken,
                     None,
                     None,
                     'Whether the kitty was picked up')
    type = property(_get_type, _set_type, None, 'The type of the kitty')
